/* 
    Neural Network Simulator

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.gom 
*/


#include <windows.h>
#include "neuron.h"

#ifndef ANIMAL_H
#define ANIMAL_H


typedef struct sensor_type
{
    double wall, food, other;
};



typedef struct animal_type
{
    animal_type *next;         // Pointer to next animal in linked list
    
    double x_size, y_size;
    double x, y, dx, dy;                // (x,y) position in world space
    double dir, ddir;                 // Facing direction
    
    double speed;
    
    double sensor_dist; // Distance from center for touch sensor
    double sensor_size; // diameter of sensor
    

    //double last_score;        // Number of plants eaten before mutation
    unsigned int food_capture;          // Number of plants eaten    
    unsigned int wall_hit;
    
    double score;
    
    unsigned num_touch_points;
    
    // Inputs to neural network
    sensor_type touch[6];


    unsigned int num_vision_points;

    sensor_type vision[30];

    double end_x[60];
    double end_y[60];
    

//    unsigned int sensor_neurons;


    unsigned int neuron_width;
    unsigned int neuron_depth;
    



    // Outputs from neural network
    double l_drive, r_drive;    // Left and right drive motors    
    
    neuron_type *neurons;       // pointer to first neuron in network
    
    unsigned int num_neurons;
    
    unsigned long l_drive_id;   // Neuron ID for the left drive output
    unsigned long r_drive_id;   // Neuron ID for the right drive output
};



void animal_draw(HDC dc, animal_type *a);
void animal_update(animal_type *a);
void animal_init(animal_type *a);
void animal_motion(double l, double r);
void animal_show_neurons(animal_type *a);

unsigned int animal_get_chromosome_size(animal_type *a);

void animal_export_chromosome(animal_type *a, double *c);

void animal_inport_chromosome(animal_type *a, double *c);

void animal_mutate_chromosome(animal_type *a, double *c, unsigned int size, 
                              double chance, double rate);


void animal_randomize( animal_type *a);

void animal_reset( animal_type *a );

#endif


