/* 
    Neural Network Simulator

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.gom 
*/


#ifndef NEURON_H
#define NEURON_H

#define MAX_SOURCES 100

/* Neuron Structure */
typedef struct neuron_type
{
    double *sources[MAX_SOURCES];
    
    double source_bias[MAX_SOURCES];    
    double source_scaler[MAX_SOURCES];    
    int source_power[MAX_SOURCES];      // integer powers only
    
    double next_val;

    double sum_weight;
    double prod_weight;
    double derv_weight;
    double intr_weight;


    double last_weight;
    double feedback_weight;
    
    double bias;

    double derivative;
    double integral;

    double last_val;
    double out;
    
    unsigned int num_sources;    
    
    unsigned long id;               // ID number for the neuron
    
    
    neuron_type *next;
};


void process_neurons( neuron_type *n );

neuron_type * get_neuron(neuron_type *n, unsigned long id);

void neuron_connect(neuron_type *n, unsigned id, double *source, 
            double bias, double scaler, int power);


void add_neuron(neuron_type **n, unsigned long id, 
                double bias, double sum, double prod, 
                double intr, double derv, double feedback);

void show_neuron(neuron_type *n, unsigned long id);

void free_neuron_list(neuron_type *n);

#endif


