/* 
    Neural Network Simulator

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2006 Jason Hunt
    nulluser@gmail.gom 
*/

#include <windows.h>

#include "animal.h"
#include "plant.h"


#ifndef WORLD_H
#define WORLD_H

/* World Structure */
typedef struct world_type
{
    // Brushes for drawing
    HBRUSH white_brush;
    HBRUSH black_brush;
    HBRUSH green_brush;
    HBRUSH blue_brush;
    HBRUSH red_brush;
    
    // Pens for drawing    
    HPEN white_pen;
    HPEN black_pen;
    HPEN green_pen;

    // All animals
    animal_type *animals;

    // All plants
    plant_type *plants;
        
    unsigned int num_plants;
    
    unsigned int num_animals;    // Current number of animals
    unsigned  cycle;          // Current cycle (used to trigger new generation)
    unsigned  generation;     // Current generation

    double average;
    double score_sum;    
    bool wrap_around;    
    bool graphics_on;
    bool running;
    
    int updates_per_display;

    // Parameters for the simulation
    double x_size, y_size;              // Size of world
        
    double mutate_chance;               // Chance from 0..1 of a mutation
    double mutate_rate;                 // Range for mutation delta
    
    int cycles_per_generation;          // Number of cycles before a new gen
    
    int initial_plants;
    int initial_animals;
        
    int animal_neuron_width;
    int animal_neuron_depth;

    double animal_size;
    double plant_size;
};


void world_setup( void );
void world_reset( void );

void world_destroy( void );
void world_draw( HDC dc );
void world_update( void );
    
    
void world_add_animal( void );

void world_free_animals( void );
void world_free_plants( void );


void get_bounding_rect(double x, double y, double x_size, double y_size, RECT &r);
int get_screen_x( double x );
int get_screen_y( double y );

void world_debug_update( void );
void world_compute_generation( void );    



void world_save( void );
void world_load( void );

#endif
